@@ -1,22 +1,65 @@
+#
+# All Rights Reserved 2020
+#
+
 from strongdoc import client
 from strongdoc import constants
 from strongdoc.proto import strongdoc_pb2_grpc, accounts_pb2
 
 from datetime import timezone
 
-class UserMetadata:
+class AccountInfo:
+    """
+    Attributes:
+        orgid: :class:`str`
+        org_email: :class:`str`
+        subscription: :class:`Subscription`
+        payments: list(:class:`Payment`)
+        org_address: :class:`str`
+        multilevel_sharing: :class:`bool`
+        sharable_orgs: list(:class:`str`)
+    """
+
+    def __init__(self, proto_account_info):
+        self.orgid = proto_account_info.orgID
+        self.org_email = proto_account_info.orgEmail
+        self.subscription = Subscription(proto_account_info.subscription)
+        self.payments = [Payment(payment) for payment in proto_account_info.payments]
+        self.org_address = proto_account_info.orgAddress
+        self.multilevel_sharing = proto_account_info.multiLevelShare
+        self.sharable_orgs = [sharable_org for sharable_org in proto_account_info.sharableOrgs]
+
+    def __repr__(self):
+        result = "\n".join(["{}: {}".format(key, str(value).replace('\n', '\n{}'.format(' '*(2+len(key))))) for key, value in self.__dict__.items()])
+        return result
+
+    def __str__(self):
+        return self.__repr__()
+
+class UserInfo:
     """
     Attributes:
-        username: :class:`str`
         userid: :class:`str`
+        name: :class:`str`
+        email: optional :class:`str`, otherwise :class:`None`
+        orgid: optional :class:`str`, otherwise :class:`None`
         is_admin: :class:`bool`
     
     """
     def __init__(self, proto_user):
-        self.username = proto_user.userName
         self.userid = proto_user.userID
+        self.name = proto_user.userName
+        try:
+            self.email = proto_user.email
+        except:
+            self.email = None
+        try:
+            self.orgid = proto_user.orgID
+        except:
+            self.orgid = None
         self.is_admin = proto_user.isAdmin
 
+
     def __repr__(self):
         result = "\n".join(["{}: {}".format(key, str(value).replace('\n', '\n{}'.format(' '*(2+len(key))))) for key, value in self.__dict__.items()])
         return result
@@ -33,7 +76,7 @@ class Subscription:
     """
     def __init__(self, proto_subscription):
         self.type = proto_subscription.type
-        self.status = proto_subscription.type
+        self.status = proto_subscription.status
 
     def __repr__(self):
         result = "\n".join(["{}: {}".format(key, str(value).replace('\n', '\n{}'.format(' '*(2+len(key))))) for key, value in self.__dict__.items()])
@@ -68,7 +111,7 @@ class Payment:
 
 
 # _register_organization creates an organization
-def _register_organization(org_name, org_addr, admin_name, admin_password, admin_email, source, source_data):
+def _register_organization(org_name, org_email, org_addr, admin_name, admin_password, admin_email, source, source_data):
     """
     Registers a new organization. A new administrator user will also be created.
     New users can be added using this administrator account.
@@ -78,6 +121,10 @@ def _register_organization(org_name, org_addr, admin_name, admin_password, admin
         The organization name to create.
     :type org_name:
         str
+    :param org_email:
+        The organization email address.
+    :type org_email:
+        str
     :param org_addr:
         The organization address.
     :type org_addr:
@@ -115,9 +162,9 @@ def _register_organization(org_name, org_addr, admin_name, admin_password, admin
     with client.connect_to_server_with_no_auth() as no_auth_conn:
         client_stub = strongdoc_pb2_grpc.StrongDocServiceStub(no_auth_conn)
 
-        request = accounts_pb2.RegisterOrganizationReq(orgName=org_name,
+        request = accounts_pb2.RegisterOrganizationReq(orgName=org_name, orgEmail=org_email,
                                                            orgAddr=org_addr, userName=admin_name,
-                                                           password=admin_password, email=admin_email, 
+                                                           password=admin_password, adminEmail=admin_email, 
                                                            source=source, sourceData=source_data)
 
         response = client_stub.RegisterOrganization(request, timeout=constants.GRPC_TIMEOUT)
@@ -160,7 +207,7 @@ def remove_organization(token, force):
         return response.success
 
 # register_user registers a user for the active organization
-def register_user(token, username, password, email, make_admin):
+def register_user(token, name, password, email, make_admin):
     """
     Registers a user for the active organization.
     This requires an administrator privilege.
@@ -170,9 +217,9 @@ def register_user(token, username, password, email, make_admin):
         The user JWT token.
     :type token:
         str
-    :param username:
-        The new user's username.
-    :type username:
+    :param name:
+        The new user's name.
+    :type name:
         str
     :param password:
         The new user's password.
@@ -198,7 +245,7 @@ def register_user(token, username, password, email, make_admin):
     with client.connect_to_server_with_auth(token) as auth_conn:
         client_stub = strongdoc_pb2_grpc.StrongDocServiceStub(auth_conn)
 
-        request = accounts_pb2.RegisterUserReq(userName=username, password=password, email=email, admin=make_admin)
+        request = accounts_pb2.RegisterUserReq(userName=name, password=password, email=email, admin=make_admin)
 
         response = client_stub.RegisterUser(request, timeout=constants.GRPC_TIMEOUT)
 
@@ -219,7 +266,7 @@ def list_users(token):
     :returns:
         A list of all the users in the organization.
     :rtype:
-        list(UserMetadata) 
+        list(UserInfo) 
     """
     with client.connect_to_server_with_auth(token) as auth_conn:
         client_stub = strongdoc_pb2_grpc.StrongDocServiceStub(auth_conn)
@@ -228,7 +275,7 @@ def list_users(token):
 
         response = client_stub.ListUsers(request, timeout=constants.GRPC_TIMEOUT)
 
-        return [UserMetadata(user) for user in response.orgUsers]
+        return [UserInfo(user) for user in response.orgUsers]
 
 # remove_user removes a user from the active organization
 def remove_user(token, userid):
@@ -329,6 +376,7 @@ def demote_user(token, userid):
 def add_sharable_org(token, orgid):
     """
     Adds another org as a sharable org.
+    This requires an administrator privilege.
 
     :param token: 
         The user JWT token.
@@ -360,6 +408,7 @@ def add_sharable_org(token, orgid):
 def remove_sharable_org(token, orgid):
     """
     Removes a sharable org.
+    This requires an administrator privilege.
 
     :param token: 
         The user JWT token.
@@ -391,6 +440,7 @@ def remove_sharable_org(token, orgid):
 def set_multilevel_sharing(token, enable):
     """
     Enables or disables multi-level sharing.
+    This requires an administrator privilege.
 
     :param token: 
         The user JWT token.
@@ -421,7 +471,8 @@ def set_multilevel_sharing(token, enable):
 
 def get_account_info(token):
     """
-    Gets subscription and payment information for the active account.
+    Gets account information such as subscription details, payments, sharable orgs, etc.
+    This requires an administrator privilege.
 
     :param token: 
         The user JWT token.
@@ -431,12 +482,10 @@ def get_account_info(token):
     :raises grpc.RpcError:
         Raised by the gRPC library to indicate non-OK-status RPC termination.
 
-    Returns
-    -------
-    subscription: :class:`Subscription`
-        The subscription information
-    payments: list(:class:`Payment`)
-        A list of payment information.
+    :returns:
+        The account information
+    :rtype:
+        AccountInfo
     """
 
     with client.connect_to_server_with_auth(token) as auth_conn:
@@ -446,6 +495,136 @@ def get_account_info(token):
 
         response = client_stub.GetAccountInfo(request, timeout=constants.GRPC_TIMEOUT)
 
-        payments = [Payment(payment) for payment in response.payments]
+        return AccountInfo(response)
 
-        return Subscription(response.subscription), payments
+def set_account_info(token, org_email, org_address):
+    """
+    Set organization account info.
+    This requires an administrator privilege.
+
+    :param token: 
+        The user JWT token.
+    :type token:
+        str
+    :param email:
+        The organization email address. Must be a valid email address.
+    :type email:
+        str
+    :param address:
+        The organization address.
+    :type address:
+        str
+
+    :raises grpc.RpcError:
+        Raised by the gRPC library to indicate non-OK-status RPC termination.
+        
+    :returns:
+        Whether the change was a success.
+    :rtype:
+        bool
+    """
+    with client.connect_to_server_with_auth(token) as auth_conn:
+        client_stub = strongdoc_pb2_grpc.StrongDocServiceStub(auth_conn)
+
+        request = accounts_pb2.SetAccountInfoReq(orgEmail=org_email, orgAddress=org_address)
+
+        response = client_stub.SetAccountInfo(request, timeout=constants.GRPC_TIMEOUT)
+
+        return response.success
+
+def get_user_info(token):
+    """
+    Gets information about the active user.
+
+    :param token: 
+        The user JWT token.
+    :type token:
+        str
+
+    :raises grpc.RpcError:
+        Raised by the gRPC library to indicate non-OK-status RPC termination.
+
+    :returns:
+        The user information
+    :rtype:
+        UserInfo
+    """
+
+    with client.connect_to_server_with_auth(token) as auth_conn:
+        client_stub = strongdoc_pb2_grpc.StrongDocServiceStub(auth_conn)
+
+        request = accounts_pb2.GetUserInfoReq()
+
+        response = client_stub.GetUserInfo(request, timeout=constants.GRPC_TIMEOUT)
+
+        return UserInfo(response)
+
+def set_user_info(token, name, email):
+    """
+    Set user info.
+
+    :param token: 
+        The user JWT token.
+    :type token:
+        str
+    :param name:
+        The user's name. Cannot be an empty string.
+    :type name:
+        str
+    :param email:
+        The user's email address. Must be a valid email address.
+    :type email:
+        str
+
+    :raises grpc.RpcError:
+        Raised by the gRPC library to indicate non-OK-status RPC termination.
+        
+    :returns:
+        Whether the change was a success.
+    :rtype:
+        bool
+    """
+
+    with client.connect_to_server_with_auth(token) as auth_conn:
+        client_stub = strongdoc_pb2_grpc.StrongDocServiceStub(auth_conn)
+
+        request = accounts_pb2.SetUserInfoReq(name=name, email=email)
+
+        response = client_stub.SetUserInfo(request, timeout=constants.GRPC_TIMEOUT)
+
+        return response.success
+
+def change_user_password(token, old_password, new_password):
+    """
+    Change user password.
+
+    :param token: 
+        The user JWT token.
+    :type token:
+        str
+    :param old_password:
+        The user's current password.
+    :type old_password:
+        str
+    :param new_password:
+        The user's new password.
+    :type new_password:
+        str
+
+    :raises grpc.RpcError:
+        Raised by the gRPC library to indicate non-OK-status RPC termination.
+        
+    :returns:
+        Whether the change was a success.
+    :rtype:
+        bool
+    """
+
+    with client.connect_to_server_with_auth(token) as auth_conn:
+        client_stub = strongdoc_pb2_grpc.StrongDocServiceStub(auth_conn)
+
+        request = accounts_pb2.ChangeUserPasswordReq(oldPassword=old_password, newPassword=new_password)
+
+        response = client_stub.ChangeUserPassword(request, timeout=constants.GRPC_TIMEOUT)
+
+        return response.success
