@@ -1,3 +1,7 @@
+#
+# All Rights Reserved 2020
+#
+
 from typing import List
 
 from strongdoc import client
@@ -21,7 +25,7 @@ def search(token, query):
     :raises grpc.RpcError:
         Raised by the gRPC library to indicate non-OK-status RPC termination.
 
-    returns:
+    :returns:
         The hit list of the search.
     :rtype:
         list(DocumentResult)
